#include <iostream>

#include <esp_log.h>

#include "kbf/driver/button.h"

using namespace std;
using namespace kbf;

driver::Button::Button(int pin, int longPressMs) :
        pin(pin),
        longPressMs(longPressMs),
        input(gpio::Input(pin, this)),
        eventTask(new rtos::Task("kbf_button_task", eventTaskHandler, this)
        ) {

    ESP_LOGI(TAG, "Button(%d, %d)", pin, longPressMs);

    input.onHigh = handleHigh;
    input.onLow  = handleLow;
}

void driver::Button::handleHigh(gpio::Input &input) {
    auto button = static_cast<driver::Button *>(input.userData);

    button->eventGroup.setBit(eventHigh);

    if (button->longPressTask) delete button->longPressTask; // NOLINT(readability-delete-null-pointer)
    button->longPressTask = new rtos::Task("kbf_button_lp" + to_string(button->pin), longPressTaskHandler, button);
}

void driver::Button::handleLow(gpio::Input &input) {
    auto button = static_cast<driver::Button *>(input.userData);

    button->eventGroup.setBit(eventLow);
    button->longPressEventGroup.setBit(eventLongRelease);
}

[[noreturn]] void driver::Button::eventTaskHandler(void *arg) {
    auto button = static_cast<driver::Button *>(arg);

    while (true) {
        button->eventGroup.waitForAny({eventLow, eventHigh});
        if (button->eventGroup.getBit(eventHigh)) {
            button->eventGroup.clearBit(eventHigh);
            if (button->onPress) button->onPress(*button);
        }
        if (button->eventGroup.getBit(eventLow)) {
            button->eventGroup.clearBit(eventLow);
            if (button->onRelease) button->onRelease(*button);
        }
    }
}

void driver::Button::longPressTaskHandler(void *arg) {
    auto button = static_cast<driver::Button *>(arg);

    if (button->longPressEventGroup.waitForBit(eventLongRelease, button->longPressMs)) {
        button->longPressEventGroup.clear();
        return;
    }
    if (button->onLongPress) button->onLongPress(*button);
    button->longPressEventGroup.waitForBit(eventLongRelease);
    if (button->onLongRelease) button->onLongRelease(*button);
}
