#include "kbf/driver/lcd.h"
#include "kbf/macros.h"

#include <string>

#include <esp_log.h>

#include "kbf.h"

using std::string;

kbf::driver::LCD::LCD(uint8_t rs, uint8_t en, uint8_t d4, uint8_t d5, uint8_t d6, uint8_t d7) : lcd{
        .write_cb = nullptr,
        .pins = {
                .rs = rs,
                .e  = en,
                .d4 = d4,
                .d5 = d5,
                .d6 = d6,
                .d7 = d7,
                .bl = HD44780_NOT_USED
        },
        .font = HD44780_FONT_5X8,
        .lines = 2,
        .backlight = true,
} {
    ESP_LOGI(TAG, "LCD(%d, %d, %d, %d, %d, %d)", rs, en, d4, d5, d6, d7);
    CHECK(hd44780_init(&lcd));
}

void kbf::driver::LCD::control(bool on, bool cursor, bool blink) {
    ESP_LOGD(TAG, "control(%d, %d, %d)", on, cursor, blink);
    CHECK(hd44780_control(&lcd, on, cursor, blink));
}

void kbf::driver::LCD::clear() {
    ESP_LOGD(TAG, "clear()");
    CHECK(hd44780_clear(&lcd));
    kbf::sleep(20);
}

void kbf::driver::LCD::move(int x, int y) {
    ESP_LOGD(TAG, "move(%d, %d)", x, y);
    CHECK(hd44780_gotoxy(&lcd, x, y));
}

template<>
void kbf::driver::LCD::print(const char *data) {
    ESP_LOGD(TAG, "print char array: %s", data);
    CHECK(hd44780_puts(&lcd, data));
}

template<>
void kbf::driver::LCD::print(string data) {
    ESP_LOGD(TAG, "print std::string: %s", data.c_str());
    CHECK(hd44780_puts(&lcd, data.c_str()));
}

template<>
void kbf::driver::LCD::print(char data) {
    ESP_LOGD(TAG, "print char: %c", data);
    CHECK(hd44780_putc(&lcd, data));
}

template<>
void kbf::driver::LCD::print(int data) {
    ESP_LOGD(TAG, "print int: %d", data);
    CHECK(hd44780_puts(&lcd, std::to_string(data).c_str()));
}

template<typename T>
void kbf::driver::LCD::print(T data) {
    ESP_LOGW(TAG, "print (template)");
    CHECK(hd44780_puts(&lcd, data));
}

void kbf::driver::LCD::println(const string& data, int lineNumber) {
    ESP_LOGD(TAG, "println: %s", data.c_str());
    move(0, lineNumber);
    auto line = data + string(16 - data.length(), ' ');
    CHECK(hd44780_puts(&lcd, line.c_str()));
}

kbf::driver::LCD::~LCD() {
    ESP_LOGI(TAG, "~LCD()");
}

void kbf::driver::LCD::setChr(int slot, const uint8_t *data) {
    ESP_LOGI(TAG, "setChr(%d, ...)", slot);
    ESP_LOG_BUFFER_HEXDUMP(TAG, data, 8, ESP_LOG_INFO);
    assert(slot >= 0 && slot < 8);

    CHECK(hd44780_upload_character(&lcd, slot, data));
}

void kbf::driver::LCD::scrollLeft() {
    hd44780_scroll_left(&lcd);
}

void kbf::driver::LCD::scrollRight() {
    hd44780_scroll_right(&lcd);
}

char kbf::driver::LCD::Chr::operator[](int slot) {
    ESP_LOGI(TAG, "Chr::operator[](%d)", slot);
    return char(8 + slot);
}
