#include "kbf/web_service.h"

#include <kbf/macros.h>

#include <utility>

using namespace kbf;

WebService::WebService(int port) : server(), port(port) {
}

WebService::~WebService() {
    if (running) {
        stop();
    }
}

void WebService::start() {
    if (running) {
        ABORT("WebService already running");
    }

    running = true;
    server.start(port);
}

void WebService::stop() {
    if (!running) {
        ESP_LOGW(TAG, "stop(): not running");
        return;
    }

    server.stop();
    running = false;
}

kbf::http::Response WebService::handleGet(const http::Request &request, void *data) {
    auto controller = static_cast<Controller *>(data);
    return controller->get(request);
}

kbf::http::Response WebService::handlePost(const http::Request &request, void *data) {
    auto controller = static_cast<Controller *>(data);
    return controller->post(request);
}

WebService::Controller::Controller(std::string path) : path(std::move(path)) {}

void WebService::Controller::responseSentHandler(const http::Response &response, void *data) {
    auto controller = static_cast<Controller *>(data);
    controller->onResponseSent(response);
}
