#include "kbf.h"
#include "kbf/driver/lcd.h"

#include <string>

#include <esp_log.h>

#include <unity.h>

using namespace kbf;
using std::string;

TEST_CASE("LCD", "[kbf_lcd]") {
    static const char *TAG  = "kbf_lcd_test";
    static const int  delay = 1000;
    ESP_LOGI(TAG, "initializing");

    auto lcd = driver::LCD(32, 33, 25, 26, 27, 13);
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing \"hello\"");
    lcd.print("hello");
    kbf::sleep(delay);

    ESP_LOGI(TAG, "cursor ON");
    lcd.control(true, true, false);
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing 1337");
    lcd.print(1337);
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing \'!\'");
    lcd.print('!');
    kbf::sleep(delay);

    ESP_LOGI(TAG, "moving cursor to 0, 1");
    lcd.move(0, 1);
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing using operator<<");
    lcd << " " << "over" << 9000;
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing newline");
    lcd << "\n" << "after \\n";
    kbf::sleep(delay);

    ESP_LOGI(TAG, "clearing");
    lcd.clear();
    kbf::sleep(delay);

    ESP_LOGI(TAG, "blinking ON");
    lcd.control(true, true, true);
    kbf::sleep(delay);

    ESP_LOGI(TAG, "printing std::string");
    string str = "happy!";
    lcd.print(str);
    kbf::sleep(delay);

    lcd.control(false, false, false);
}

TEST_CASE("LCD custom characters & scrolling", "[kbf_lcd]") {
    uint8_t chars[8][8] = {
            {0x00, 0x00, 0x0A, 0x1F, 0x1F, 0x0E, 0x04, 0x00},
            {0x00, 0x00, 0x0A, 0x15, 0x11, 0x0A, 0x04, 0x00},
            {0x00, 0x0A, 0x15, 0x0A, 0x00, 0x11, 0x11, 0x1F},
            {0x15, 0x0A, 0x15, 0x0A, 0x15, 0x0A, 0x15, 0x0A},

            {0x1F, 0x1F, 0x15, 0x00, 0x00, 0x11, 0x1B, 0x1F},
            {0x1F, 0x1F, 0x15, 0x0A, 0x0E, 0x15, 0x1B, 0x1F},
            {0x1F, 0x15, 0x0A, 0x15, 0x1F, 0x0E, 0x0E, 0x00},
            {0x0A, 0x15, 0x0A, 0x15, 0x0A, 0x15, 0x0A, 0x15}
    };

    auto lcd = driver::LCD(32, 33, 25, 26, 27, 13);

    for (int i = 0; i < 8; i++) {
        lcd.setChr(i, chars[i]);
    }

    for (int i = 0; i < 4; i++) {
        lcd << lcd.chr[i];
    }
    lcd.move(0, 1);
    for (int i = 4; i < 8; i++) {
        lcd << lcd.chr[i];
    }

    for (int i = 0; i < 24; i++) {
        kbf::sleep(300);
        if (i < 12) {
            lcd.scrollRight();
        } else {
            lcd.scrollLeft();
        }
    }

    for (int i = 0; i < 8; i++) {
        kbf::sleep(300);
        if (i < 4) {
            lcd.scrollLeft();
        } else {
            lcd.scrollRight();
        }
    }
}

TEST_CASE("LCD println", "[kbf_lcd]") {
    static const int delay = 1000;

    auto lcd = driver::LCD(32, 33, 25, 26, 27, 13);
    kbf::sleep(delay);

    lcd.clear();
    lcd << "1234567890abcdef";
    lcd.move(0, 1);
    lcd << "1234567890abcdef";

    kbf::sleep(delay);
    lcd.println("world!", 1);
    kbf::sleep(delay);
    lcd.println("hello, ", 0);
}
