#include <esp_log.h>

#include <kbf.h>
#include <kbf/rtos.h>

#include <unity.h>

using namespace kbf;

static const int DELAY = 500;
enum {
    TEST_EVENT_QUIT = 0,
    TEST_EVENT_RESET,
    TEST_EVENT_TRIGGER_1,
    TEST_EVENT_FINISHED,
    TEST_EVENT_TRIGGER_2,
};

static const constexpr char *const TAG = "test_event_group";

void trigger_1(void *data) {
    auto eventGroup = static_cast<rtos::EventGroup *>(data);
    while (!eventGroup->getBit(TEST_EVENT_QUIT)) {
        kbf::sleep(DELAY);
        eventGroup->setBit(TEST_EVENT_TRIGGER_1);
        eventGroup->waitForAny({TEST_EVENT_QUIT, TEST_EVENT_RESET});
        if (eventGroup->getBit(TEST_EVENT_QUIT)) break;
        eventGroup->clear();
    }
    eventGroup->setBit(TEST_EVENT_FINISHED);
}

void trigger_2(void *data) {
    auto eventGroup = static_cast<rtos::EventGroup *>(data);
    kbf::sleep(DELAY * 2);
    TEST_ASSERT_FALSE((*eventGroup)[TEST_EVENT_QUIT])
    eventGroup->setBit(TEST_EVENT_TRIGGER_2);
    eventGroup->waitForBit(TEST_EVENT_QUIT);
    eventGroup->clearBit(TEST_EVENT_TRIGGER_2);
}

TEST_CASE("RTOS EventGroup", "[kbf_rtos]") {
    auto eventGroup = rtos::EventGroup();

    ESP_LOGI(TAG, "test trigger1");
    auto task = rtos::Task("trigger1", trigger_1, &eventGroup);
    TEST_ASSERT_TRUE(eventGroup.waitForBit(TEST_EVENT_TRIGGER_1, DELAY * 2))

    ESP_LOGI(TAG, "test trigger1 timeout");
    eventGroup.setBit(TEST_EVENT_RESET);
    kbf::sleep(100);
    TEST_ASSERT_FALSE(eventGroup.waitForBit(TEST_EVENT_TRIGGER_1, (DELAY / 2) + 100))
    TEST_ASSERT_FALSE(eventGroup[TEST_EVENT_TRIGGER_1])
    kbf::sleep((DELAY / 2) + 100);
    TEST_ASSERT_TRUE(eventGroup[TEST_EVENT_TRIGGER_1])

    ESP_LOGI(TAG, "test double trigger timeout");
    eventGroup.setBit(TEST_EVENT_RESET);
    kbf::sleep(100);
    TEST_ASSERT_FALSE(eventGroup.waitForAll({TEST_EVENT_TRIGGER_1, TEST_EVENT_TRIGGER_2}, DELAY * 3))
    TEST_ASSERT_TRUE(eventGroup[TEST_EVENT_TRIGGER_1])
    TEST_ASSERT_FALSE(eventGroup[TEST_EVENT_TRIGGER_2])

    ESP_LOGI(TAG, "test double trigger");
    eventGroup.setBit(TEST_EVENT_RESET);
    kbf::sleep(100);
    auto other = rtos::Task("trigger2", trigger_2, &eventGroup);
    TEST_ASSERT_TRUE(eventGroup.waitForAll({TEST_EVENT_TRIGGER_1, TEST_EVENT_TRIGGER_2}, DELAY * 3))
    TEST_ASSERT_TRUE(eventGroup[TEST_EVENT_TRIGGER_1])
    TEST_ASSERT_TRUE(eventGroup[TEST_EVENT_TRIGGER_2])

    ESP_LOGI(TAG, "test finish");
    TEST_ASSERT_FALSE(eventGroup[TEST_EVENT_FINISHED])
    eventGroup.setBit(TEST_EVENT_QUIT);
    kbf::sleep(100);
    TEST_ASSERT_TRUE(eventGroup[TEST_EVENT_FINISHED])
    TEST_ASSERT_FALSE(eventGroup[TEST_EVENT_TRIGGER_2])
}
