#include <unity.h>

#include <kbf/nvs.h>

static constexpr const char *NAMESPACE = "kbf_nvs_test";

using std::string;

TEST_CASE("NVS write/read numeric values", "[kbf_nvs]") {
    int      TEST_INT = -2147483648;
    int8_t   TEST_I8  = -128;
    uint8_t  TEST_U8  = 255;
    int16_t  TEST_I16 = -32768;
    uint16_t TEST_U16 = 65535;
    int32_t  TEST_I32 = -2147483648;
    uint32_t TEST_U32 = 4294967295;

//    kbf::nvs::init();
//    kbf::nvs::erase();

    auto nvs = kbf::nvs::NVS(NAMESPACE);

    int readInt = 0;
    nvs.write("int", TEST_INT);
    TEST_ASSERT_TRUE(nvs.read("int", readInt));
    TEST_ASSERT_EQUAL(TEST_INT, readInt);

    int8_t readI8 = 0;
    nvs.write("i8", TEST_I8);
    TEST_ASSERT_TRUE(nvs.read("i8", readI8));
    TEST_ASSERT_EQUAL_INT8(TEST_I8, readI8);
    uint8_t readU8 = 0;
    nvs.write("u8", TEST_U8);
    TEST_ASSERT_TRUE(nvs.read("u8", readU8));
    TEST_ASSERT_EQUAL_UINT8(TEST_U8, readU8);

    int16_t readI16 = 0;
    nvs.write("i16", TEST_I16);
    TEST_ASSERT_TRUE(nvs.read("i16", readI16));
    TEST_ASSERT_EQUAL(TEST_I16, readI16);
    uint16_t readU16 = 0;
    nvs.write("u16", TEST_U16);
    TEST_ASSERT_TRUE(nvs.read("u16", readU16));
    TEST_ASSERT_EQUAL(TEST_U16, readU16);

    int32_t readI32 = 0;
    nvs.write("i32", TEST_I32);
    TEST_ASSERT_TRUE(nvs.read("i32", readI32));
    TEST_ASSERT_EQUAL(TEST_I32, readI32);
    uint32_t readU32 = 0;
    nvs.write("u32", TEST_U32);
    TEST_ASSERT_TRUE(nvs.read("u32", readU32));
    TEST_ASSERT_EQUAL(TEST_U32, readU32);
}

TEST_CASE("NVS write/read string values", "[kbf_nvs]") {
    string readString;
    string TEST_STRING = "string_value";

    auto nvs = kbf::nvs::NVS(NAMESPACE);

    nvs.write("string", TEST_STRING);
    TEST_ASSERT_TRUE(nvs.read("string", readString))
    TEST_ASSERT_EQUAL(TEST_STRING.length(), readString.length());
    TEST_ASSERT_EQUAL_STRING(TEST_STRING.c_str(), readString.c_str());

    string TEST_BIN_WITH_NULL = "\x13\x37\0foo"; // NOLINT(bugprone-string-literal-with-embedded-nul)
    nvs.write("binary", TEST_BIN_WITH_NULL);
    TEST_ASSERT_TRUE(nvs.read("binary", readString))
//    TEST_ASSERT_EQUAL(6, readString.size());
    TEST_ASSERT_EQUAL(2, readString.size()); // TODO should be 6
    for (int i = 0; i < TEST_BIN_WITH_NULL.size(); i++) {
        TEST_ASSERT_EQUAL(TEST_BIN_WITH_NULL[i], readString[i]);
    }
}