#include <kbf/wifi.h>
#include <kbf/http/client.h>
#include <kbf/web_service.h>

#include <unity.h>

using namespace kbf;

class CounterController : public WebService::Controller {
public:
    CounterController() : Controller("/counter") {}

    http::Response get(const http::Request &request) override {
        return http::Response(std::to_string(counter));
    }

    void onResponseSent(const http::Response &response) override {
        counter++;
    }

private:
    int counter = 0;
};

class EchoController : public WebService::Controller {
public:
    EchoController() : Controller("/echo") {};

    http::Response get(const http::Request &request) override {
        return http::Response(request.query.at("str"));
    }

    http::Response post(const http::Request &request) override {
        auto requestJson = request.json();
        return http::Response(requestJson.find("str")->get<string>());
    }
};

TEST_CASE("WebService", "[kbf_web_service]") {
    using namespace kbf;

    wifi::start();

    auto webService = WebService();
    webService.attach<CounterController>();
    webService.attach<EchoController>();
    webService.start();

    auto client   = http::Client();

    auto response = client.get("http://localhost/counter");
    TEST_ASSERT_EQUAL(200, response->status);
    TEST_ASSERT_EQUAL_STRING("0", response->body.c_str());

    response = client.get("http://localhost/counter");
    TEST_ASSERT_EQUAL_STRING("1", response->body.c_str());

    response = client.post("http://localhost/counter", {{"str", "notallowed"}});
    TEST_ASSERT_EQUAL(405, response->status);
    TEST_ASSERT_EQUAL_STRING("method not allowed", response->body.c_str());

    response = client.get("http://localhost/echo?str=foo");
    TEST_ASSERT_EQUAL(200, response->status);
    TEST_ASSERT_EQUAL_STRING("foo", response->body.c_str());

    response = client.post("http://localhost/echo", {{"str", "bar"}});
    TEST_ASSERT_EQUAL(200, response->status);
    TEST_ASSERT_EQUAL_STRING("bar", response->body.c_str());

    webService.stop();
    wifi::stop();
}